#!perl -T

use 5.006;
use strict;
use warnings;
use Test::More;

plan tests => 1;

BEGIN {
    use_ok('Perl::Tokenizer') || print "Bail out!\n";
}

diag("Testing Perl::Tokenizer $Perl::Tokenizer::VERSION, Perl $], $^X");
